/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.javable.cese.models;

import java.io.Serializable;

import com.javable.cese.templates.AbstractAPModel;

/**
 * The Beeler-Reuter Model of Membrane Action Potentials of Mammalian
 * Ventricular Myocytes, 1977. Detailed list of equations and model description
 * are provided in
 * <p>
 * J Physiol 1977;268:177-210.
 */

public class BeelerReuterModel_1977 extends AbstractAPModel implements
        Serializable {

    /*
     * The beginning here lists all the variables and defines them (this code
     * uses all global variables)
     */

    /* Cell Geometry */
    private double l = 0.01; // Length of the cell (cm)

    public synchronized void setl(double sl) {
        l = sl;
        getCellGeometry();
    }

    public double getl() {
        return l;
    }

    private double a = 0.0011; // Radius of the cell (cm)

    public synchronized void seta(double sa) {
        a = sa;
        getCellGeometry();
    }

    public double geta() {
        return a;
    }

    private double vcell; // Cell volume (uL)

    public double getvcell() {
        return vcell;
    }

    private double ageo; // Geometric membrane area (cm^2)

    public double getageo() {
        return ageo;
    }

    private double acap; // Capacitive membrane area (cm^2)

    public double getacap() {
        return acap;
    }

    private double vmyo; // Myoplasm volume (uL)

    public double getvmyo() {
        return vmyo;
    }

    private double vmito; // Mitochondria volume (uL)

    public double getvmito() {
        return vmito;
    }

    private double vcleft; // Cleft volume (uL)

    public double getvcleft() {
        return vcleft;
    }

    /* Terms for Solution of Conductance and Reversal Potential */
    public static final double R = 8314; // Universal Gas Constant (J/kmol*K)
    public static final double frdy = 96485; // Faraday's Constant (C/mol)

    private double temp = 310; // Temperature (K)

    public synchronized void settemp(double stemp) {
        temp = stemp;
    }

    public double gettemp() {
        return temp;
    }

    /* Ion Valences */
    public static final double zna = 1; // Na valence
    public static final double zk = 1; // K valence
    public static final double zca = 2; // Ca valence

    /* Ion Concentrations */
    private double cai = 0.0002; // Intracellular Ca Concentration (mM)

    public synchronized void setcai(double scai) {
        cai = scai;
    }

    public double getcai() {
        return cai;
    }

    /* Myoplasmic Na Ion Concentration Changes */
    private double naiont; // Total Na Ion Flow (mM/ms)

    public double getnaiont() {
        return naiont;
    }

    private double dnai; // Change in Intracellular Na Concentration (mM)

    public double getdnai() {
        return dnai;
    }

    /* Myoplasmic K Ion Concentration Changes */
    private double kiont; // Total K Ion Flow (mM/ms)

    public double getkiont() {
        return kiont;
    }

    private double dki; // Change in Intracellular K Concentration (mM)

    public double getdki() {
        return dki;
    }

    /* Myoplasmic Ca Ion Concentration Changes */
    private double caiont; // Total Ca Ion Flow (mM/ms)

    public double getcaiont() {
        return caiont;
    }

    private double dcai; // Change in myoplasmic Ca concentration (mM)

    public double getdcai() {
        return dcai;
    }

    /* Fast Sodium Current (time dependent) */
    private double ina; // Fast Na Current (uA/uF)

    public synchronized void setina(double sina) {
        ina = sina;
    }

    public double getina() {
        return ina;
    }

    private double gna = 4.0; // Max. Conductance of the Na Channel (mS/uF)

    public synchronized void setgna(double sgna) {
        gna = sgna;
    }

    public double getgna() {
        return gna;
    }

    private double gnac = 0.003; // Conductance of the Na Channel (mS/uF)

    public synchronized void setgnac(double sgnac) {
        gnac = sgnac;
    }

    public double getgnac() {
        return gnac;
    }

    private double ena = 50.0; // Reversal Potential of Na (mV)

    public synchronized void setena(double sena) {
        ena = sena;
    }

    public double getena() {
        return ena;
    }

    private double ah; // Na alpha-h rate constant (ms^-1)

    public double getah() {
        return ah;
    }

    private double bh; // Na beta-h rate constant (ms^-1)

    public double getbh() {
        return bh;
    }

    private double aj; // Na alpha-j rate constant (ms^-1)

    public double getaj() {
        return aj;
    }

    private double bj; // Na beta-j rate constant (ms^-1)

    public double getbj() {
        return bj;
    }

    private double am; // Na alpha-m rate constant (ms^-1)

    public double getam() {
        return am;
    }

    private double bm; // Na beta-m rate constant (ms^-1)

    public double gebm() {
        return bm;
    }

    private double h = 0.988; // Na inactivation h

    public double geth() {
        return h;
    }

    private double j = 0.975; // Na inactivation j

    public double getj() {
        return j;
    }

    private double m = 0.011; // Na activation m

    public double getm() {
        return m;
    }

    private double scgna = 1;

    // Scaling factor for Max. Conductance of the Na Channel
    public synchronized void setscgna(double sscgna) {
        scgna = sscgna;
    }

    public double getscgna() {
        return scgna;
    }

    /* Inward Ca Current */
    private double ica; // Total current through the Inward Ca channel (uA/uF)

    public synchronized void setica(double sica) {
        ica = sica;
    }

    public double getica() {
        return ica;
    }

    private double gca = 0.09; // Max. Ca current through Ca channel (uA/uF)

    public synchronized void setgca(double sgca) {
        gca = sgca;
    }

    public double getgca() {
        return gca;
    }

    private double eca; // Reversal Potential of Inward Ca Current (mV)

    public synchronized void seteca(double seca) {
        eca = seca;
    }

    public double geteca() {
        return eca;
    }

    private double ad; // Ca alpha-d rate constant (ms^-1)

    public double getad() {
        return ad;
    }

    private double bd; // Ca beta-d rate constant (ms^-1)

    public double getbd() {
        return bd;
    }

    private double d = 0.003; // Voltage dependent activation gate d

    public double getd() {
        return d;
    }

    private double af; // Ca alpha-f rate constant (ms^-1)

    public double getaf() {
        return af;
    }

    private double bf; // Ca beta-f rate constant (ms^-1)

    public double getbf() {
        return bf;
    }

    private double f = 0.994; // Voltage dependent inactivation gate f

    public double getf() {
        return f;
    }

    private double scgca = 1;

    // Scaling factor for Max. Conductance of the Inward Ca channel
    public synchronized void setscgca(double sscgca) {
        scgca = sscgca;
    }

    public double getscgca() {
        return scgca;
    }

    /* Time-dependent Potassium Current */
    private double ikt; // Time-dependent K Current (uA/uF)

    public synchronized void setikt(double sikt) {
        ikt = sikt;
    }

    public double getikt() {
        return ikt;
    }

    private double gkt;

    // Channel Conductance of Time-dependent K Current (mS/uF)
    public synchronized void setgkt(double sgkt) {
        gkt = sgkt;
    }

    public double getgkt() {
        return gkt;
    }

    private double axt; // K alpha-xt rate constant (ms^-1)

    public double getaxt() {
        return axt;
    }

    private double bxt; // K beta-xt rate constant (ms^-1)

    public double getbxt() {
        return bxt;
    }

    private double xt = 0.00539; // Time-dependent K activation

    public double getxt() {
        return xt;
    }

    private double r; // K time-independent inactivation r

    public double getr() {
        return r;
    }

    private double scgkt = 1;

    // Scale factor for Channel Conductance of Time-dependent K Current
    public synchronized void setscgkt(double sscgkt) {
        scgkt = sscgkt;
    }

    public double getscgkt() {
        return scgkt;
    }

    /* Potassium Current (time-independent) */
    private double iki; // Time-independent K current (uA/uF)

    public synchronized void setiki(double siki) {
        iki = siki;
    }

    public double getiki() {
        return iki;
    }

    private double gki;

    // Channel Conductance of Time Independent K Current (mS/uF)
    public synchronized void setgki(double sgki) {
        gki = sgki;
    }

    public double getgki() {
        return gki;
    }

    private double aki; // K alpha-ki rate constant (ms^-1)

    public double getaki() {
        return aki;
    }

    private double bki; // K beta-ki rate constant (ms^-1)

    public double getbki() {
        return bki;
    }

    private double kin; // K inactivation

    public double getkin() {
        return kin;
    }

    private double scgki = 1;

    // Scaling factor for Channel Conductance of Time Independent K Current
    public synchronized void setscgki(double sscgki) {
        scgki = sscgki;
    }

    public double getscgki() {
        return scgki;
    }

    /* ** CONSTRUCTOR ** */
    public BeelerReuterModel_1977() {
        super();
        getCellGeometry();
    }

    protected void setCurrents() {
        currentINa();
        currentICa();
        currentIKt();
        currentIKi();
        currentIt();
    }

    protected void setConcentrations() {
        concentrationCai();
    }

    protected void doSimulate() {
        setCurrents();
        setConcentrations();

        vnew = v - it * dt;
        dvdtnew = (vnew - v) / dt;
        t = t + dt;
        v = vnew;
        dvdt = dvdtnew;
    }

    /** ***************************************************** */
    /**
     * Cell Geometry
     */
    protected void getCellGeometry() {
        vcell = 1000 * Math.PI * a * a * l;
        ageo = 2 * Math.PI * a * a + 2 * Math.PI * a * l;
        acap = ageo * 2;
        vmyo = vcell * 0.68;
        vmito = vcell * 0.26;
        vcleft = vcell * 0.12 / 0.88;
    }

    /* Functions that describe the currents begin here */

    /**
     * Calculates Fast Na Current
     */
    protected void currentINa() {
        gna = 4.0 * scgna;

        if (v == -47.0) {
            am = 10.0;
        } else {
            am = (-1.0 * (v + 47.0)) / (Math.exp(-0.1 * (v + 47.0)) - 1.0);
        }

        bm = 40.0 * Math.exp(-0.056 * (v + 72.0));

        ah = 0.126 * Math.exp(-0.25 * (v + 77.0));
        bh = 1.7 / (Math.exp(-0.082 * (v + 22.5)) + 1.0);

        aj = (0.055 * Math.exp(-0.25 * (v + 78.0)))
                / (Math.exp(-0.2 * (v + 78.0)) + 1.0);
        bj = 0.3 / (Math.exp(-0.1 * (v + 32.0)) + 1.0);

        if (t == 0) // Steady State Values
        {
            h = ah / (ah + bh);
            j = aj / (aj + bj);
            m = am / (am + bm);
        } else {
            h = ah / (ah + bh) - ((ah / (ah + bh)) - h)
                    * Math.exp(-dt / (1 / (ah + bh)));
            j = aj / (aj + bj) - ((aj / (aj + bj)) - j)
                    * Math.exp(-dt / (1 / (aj + bj)));
            m = am / (am + bm) - ((am / (am + bm)) - m)
                    * Math.exp(-dt / (1 / (am + bm)));
        }

        ina = (gna * m * m * m * h * j + gnac) * (v - ena);
    }

    /**
     * Calculates Inward Ca Current
     */
    protected void currentICa() {
        gca = 0.09 * scgca;
        eca = 7.7 - 13.0287 * Math.log(cai);

        ad = 0.095 * Math.exp(-0.01 * (v - 5.0))
                / (1 + Math.exp(-0.072 * (v - 5.0)));
        bd = 0.07 * Math.exp(-0.017 * (v + 44.0))
                / (1 + Math.exp(0.05 * (v + 44.0)));

        af = 0.012 * Math.exp(-0.008 * (v + 28.0))
                / (1 + Math.exp(0.15 * (v + 28.0)));
        bf = 0.0065 * Math.exp(-0.02 * (v + 30.0))
                / (1 + Math.exp(-0.2 * (v + 30.0)));

        if (t == 0) // Steady State Values
        {
            d = ad / (ad + bd);
            f = af / (af + bf);
        } else {
            d = ad / (ad + bd) - ((ad / (ad + bd)) - d)
                    * Math.exp(-dt / (1 / (ad + bd)));
            f = af / (af + bf) - ((af / (af + bf)) - f)
                    * Math.exp(-dt / (1 / (af + bf)));
        }

        ica = gca * d * f * (v - eca);
    }

    /**
     * Calculates Time-dependent K Current
     */
    protected void currentIKt() {
        gkt = scgkt;

        r = 0.8 * ((Math.exp(0.04 * (v + 77.0)) - 1.0) / Math
                .exp(0.04 * (v + 35.0)));

        axt = 0.0005 * Math.exp(0.083 * (v + 50.0))
                / (1.0 + Math.exp(0.057 * (v + 50.0)));
        bxt = 0.0013 * Math.exp(-0.06 * (v + 20.0))
                / (1.0 + Math.exp(-0.04 * (v + 20.0)));

        if (t == 0) // Steady State Values
        {
            xt = axt / (axt + bxt);
        } else {
            xt = axt / (axt + bxt) - ((axt / (axt + bxt)) - xt)
                    * Math.exp(-dt / (1 / (axt + bxt)));
        }

        ikt = gkt * xt * r;
    }

    /**
     * Calculates Time-Independent K Current
     */
    protected void currentIKi() {
        gki = scgki;

        if (v == -23.0) {
            aki = 4.0
                    * (Math.exp(0.04 * (v + 85.0)) - 1.0)
                    / (Math.exp(0.08 * (v + 53.0)) + Math
                            .exp(0.04 * (v + 53.0)));
            bki = 5.0;
        } else {
            aki = 4.0
                    * (Math.exp(0.04 * (v + 85.0)) - 1.0)
                    / (Math.exp(0.08 * (v + 53.0)) + Math
                            .exp(0.04 * (v + 53.0)));
            bki = 0.2 * (v + 23.0) / (1.0 - Math.exp(-0.04 * (v + 23.0)));
        }

        kin = 0.35 * (aki + bki);

        iki = gki * kin;
    }

    /**
     * Calculates Total Current
     * 
     * Total sum of currents is calculated here, if the time is between stimtime =
     * 0 and stimtime = tstim, a stimulus is applied
     */
    protected void currentIt() {
        naiont = ina;
        kiont = ikt + iki;
        caiont = ica;

        if ((t >= tstim) && (t < (tstim + dt))) {
            stimtime = 0;
            tstim = tstim + bcl;
            dvdtmax = false;
        }

        if ((stimtime >= 0) && (stimtime < stl)) {
            it = st + naiont + kiont + caiont;
        } else {
            it = naiont + kiont + caiont;
        }

        stimtime = stimtime + dt;
    }

    /* Functions that calculate intracellular ion concentrations begins here */

    /**
     * Calculates new myoplasmic Ca ion concentration
     */
    protected void concentrationCai() {
        dcai = dt * ((-0.0001 * caiont) + 0.07 * (0.0001 - cai));
        cai = dcai + cai;
    }

}