/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.gui;

import java.awt.Font;
import java.awt.Frame;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.Hashtable;

import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import ch.cern.lhcb.xmleditor.specific.XmlDisplay;

/**
 * A dialog windows that allows configuration of attribute displaying in an
 * XmlTree
 * 
 * @author Sebastien Ponce
 */
public class AttributeDialog extends AbstractDialog implements ItemListener {

    /**
     * the layout manager of the panel
     */
    private GridBagLayout layout;

    /**
     * a hashtable of element names that currently have an associated checkbox
     * indexed through their checkbox
     */
    private Hashtable displayBoxes;

    /**
     * constructor : builds the dialog
     * 
     * @param owner the frame where this dialog should appear
     * @param xmlDisplay the XmlDisplay that must be used
     */
    public AttributeDialog(Frame owner, XmlDisplay xmlDisplay) {
        // calls the super constructor
        super(owner, "Attribute Displaying", xmlDisplay);
        // initializes the hashtable
        this.displayBoxes = new Hashtable();
        // adds a layout to the panel where details are displayed
        layout = new GridBagLayout();
        panel.setLayout(layout);
    }

    /**
     * builds the panel for a given element name of a given XmlDisplay by
     * displaying some subelements
     * 
     * @param name the name of the element for which the panel should be built
     * @param xmlDisplay the XmlDisplay dealing with the dtd in which the
     *        element name was declared
     */
    protected void buildPanel(String name, XmlDisplay xmlDisplay) {
        // purge the hashtable
        displayBoxes.clear();
        // stores the current xmlDisplay
        this.xmlDisplay = xmlDisplay;
        // retrieves the attributes
        String[] attributes = xmlDisplay.getAttributeNames(name);
        // creates a GridBagConstraint object that will be used for display
        GridBagConstraints c = new GridBagConstraints();
        // makes titles for each column
        JLabel attrLabel = new JLabel("Attribute's Name");
        Font font = new Font("dialog", Font.PLAIN, 16);
        attrLabel.setFont(font);
        attrLabel.setAlignmentX(SwingConstants.LEFT);
        c.weightx = 1;
        c.gridx = 0;
        c.gridy = 0;
        c.anchor = GridBagConstraints.WEST;
        c.insets = new Insets(0, 0, 5, 0);
        layout.setConstraints(attrLabel, c);
        panel.add(attrLabel);
        JLabel cbLabel = new JLabel("displayed");
        cbLabel.setFont(font);
        cbLabel.setAlignmentX(SwingConstants.RIGHT);
        c.weightx = 0;
        c.gridx = 1;
        layout.setConstraints(cbLabel, c);
        panel.add(cbLabel);
        c.insets = new Insets(0, 10, 0, 0);
        // for each attribute, displays it
        for (int i = 0; i < attributes.length; i++) {
            // creates and display label
            JLabel label = new JLabel(attributes[i]);
            label.setAlignmentX(SwingConstants.LEFT);
            c.weightx = 1;
            c.gridx = 0;
            c.gridy = i + 2;
            c.anchor = GridBagConstraints.WEST;
            layout.setConstraints(label, c);
            panel.add(label);
            // creates and display checkbox
            JCheckBox checkBox = new JCheckBox();
            displayBoxes.put(checkBox, attributes[i]);
            checkBox.setSelected(xmlDisplay.isAttributeDisplayed(name,
                    attributes[i]));
            checkBox.addItemListener(this);
            c.weightx = 0;
            c.gridx = 1;
            c.anchor = GridBagConstraints.CENTER;
            layout.setConstraints(checkBox, c);
            panel.add(checkBox);
        }
    }

    /**
     * called each time an item event occurs
     * 
     * @param e the ItemEvent
     */
    public void itemStateChanged(ItemEvent e) {
        JCheckBox checkBox = (JCheckBox) e.getSource();
        xmlDisplay.setAttributeDisplayed((String) box.getSelectedItem(),
                (String) displayBoxes.get(checkBox), checkBox.isSelected());
    }

}