/*
 * This is part of XmlEditor, a graphical editor for XML files
 * Copyright (C) 2001 CERN - Sebastien Ponce
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 * 
 * Contacts :
 *   sebastien.ponce@polytechnique.org
 *   http://lhcb-comp.web.cern.ch/lhcb-comp/
 *   http://www.cern.ch
 */

package ch.cern.lhcb.xmleditor.transfer;

import java.awt.Toolkit;
import java.awt.datatransfer.Clipboard;
import java.awt.datatransfer.ClipboardOwner;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.UnsupportedFlavorException;
import java.io.IOException;
import java.io.InputStream;
import java.io.Reader;
import java.io.StringReader;
import java.util.Vector;

import javax.swing.tree.TreePath;

import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;

import ch.cern.lhcb.xmleditor.gui.MessageDisplayer;
import ch.cern.lhcb.xmleditor.gui.XmlTree;
import ch.cern.lhcb.xmleditor.gui.XmlTreeNode;
import ch.cern.lhcb.xmleditor.undo.CompoundEdit;
import ch.cern.lhcb.xmleditor.xerces.CustomDOMParser;
import ch.cern.lhcb.xmleditor.xerces.ElementImpl;
import ch.cern.lhcb.xmleditor.xerces.ErrorHandler;
import ch.cern.lhcb.xmleditor.xerces.TreeElement;

/**
 * this class deals with the Cut and Paste facilities of the xmleditor
 * 
 * @author Sebastien Ponce
 */
public class CutAndPaste implements ClipboardOwner {

    /**
     * the underlying tree
     */
    private XmlTree tree;

    /**
     * the underlying MessageDisplayer
     */
    private MessageDisplayer msgDis;

    /**
     * constructor
     * 
     * @param tree the underlying jTree
     * @param msgDis a messageDisplayer for error display
     */
    public CutAndPaste(XmlTree tree, MessageDisplayer msgDis) {
        super();
        this.tree = tree;
        this.msgDis = msgDis;
    }

    /**
     * Notifies this object that it is no longer the owner of the contents of
     * the clipboard.
     * 
     * @param clipboard the clipboard that is no longer owned
     * @param contents the contents which this owner had placed on the clipboard
     */
    public void lostOwnership(Clipboard clipboard, Transferable contents) {
        // nothing to do when ownership is lost
    }

    /**
     * sets the content of the clipboard to the current selected node, if there
     * is one. If remove is true, also removes the node
     * 
     * @param remove if true, removes the selected node
     */
    private void setClipboardContent(boolean remove) {
        // gets the clipboard
        Clipboard cb = tree.getToolkit().getSystemClipboard();
        // gets the selected node
        TreePath[] treePaths = tree.getSelectionPaths();
        Vector nodes = new Vector();
        for (int i = 0; i < treePaths.length; i++) {
            nodes.add(((XmlTreeNode) treePaths[i].getLastPathComponent())
                    .getNode());
        }
        // sets the content of the clipboard
        cb.setContents(new TransferableNode(nodes), this);
        // eventually removes the node
        if (remove) {
            // begins a compoundedit for undo management since several nodes
            // are removed at once
            CompoundEdit edit = new CompoundEdit(tree.getTreeView().getEditor());
            tree.getTreeView().getEditor().getUndoManager().addEdit(edit);
            // removes every node
            for (int i = 0; i < treePaths.length; i++) {
                // for each node
                TreeElement node = (TreeElement) nodes.elementAt(i);
                // gets the parent
                ElementImpl parent = (ElementImpl) node.getParentNode();
                // removes the node from the document
                parent.undoableRemoveChild(node);
            }
            // closes the compound edit
            edit.end();
        }
    }

    /**
     * called each time a cut has to be performed in this tree. The selected
     * item will be cut and placed into the clipboard.
     */
    public void cut() {
        this.setClipboardContent(true);
    }

    /**
     * called each time a copy has to be performed in this tree. The selected
     * item will be copied into the clipboard.
     */
    public void copy() {
        this.setClipboardContent(false);
    }

    /**
     * builds a node from an xml fragment
     * 
     * @param source an InputSource providing the xml fragment
     * @return the new Document or null if the source was not valid xml
     */
    private Document getNodeFromInputSource(InputSource source) {
        DOMParser parser = CustomDOMParser.getParser();
        try {
            parser.parse(source);
        } catch (java.io.IOException e) {
            return null;
        } catch (org.xml.sax.SAXException e) {
            return null;
        }
        ErrorHandler errorHandler = (ErrorHandler) parser.getErrorHandler();
        if (errorHandler.errorsExisting()) {
            errorHandler.displayErrors(msgDis);
        }
        return parser.getDocument();
    }

    /**
     * called each time something has to be pasted in this tree from the
     * clipborad
     * 
     * @param parent the element where to paste the content of the clipboard
     */
    private void pasteIntoTree(ElementImpl parent) {
        // gets the clipboard
        Clipboard cb = Toolkit.getDefaultToolkit().getSystemClipboard();
        // gets the transferable data
        Transferable tr = cb.getContents(this);
        // This is a work around for a bug of the motif clipboard
        // This has to be removed as soon as the bug is corrected -- FIXME
        boolean stringFlavorInMotif = false;
        if (tr.getClass().getName().equals("sun.awt.motif.X11Selection")) {
            stringFlavorInMotif = true;
        }
        // begins a compoundedit for undo management since several nodes
        // are removed at once
        CompoundEdit edit = new CompoundEdit(tree.getTreeView().getEditor());
        tree.getTreeView().getEditor().getUndoManager().addEdit(edit);
        try {
            // checks if the local flavor is supported
            if (tr.isDataFlavorSupported(TransferableNode.LocalFlavor)) {
                // clears the selection in the tree
                tree.clearSelection();
                // gets the nodes and clones them
                Vector nodes = (Vector) tr
                        .getTransferData(TransferableNode.LocalFlavor);
                for (int i = 0; i < nodes.size(); i++) {
                    TreeElement node = (TreeElement) nodes.elementAt(i);
                    TreeElement newNode = (TreeElement) node.cloneNode(true);
                    // does the paste
                    parent.pasteNode(newNode, i == nodes.size() - 1);
                }
            } else if (tr.isDataFlavorSupported(DataFlavor.stringFlavor)
            // This is a work around for a bug of the motif clipboard
                    // This has to be removed as soon as the bug is corrected --
                    // FIXME
                    || stringFlavorInMotif) {
                Object stream = tr.getTransferData(DataFlavor.stringFlavor);
                // deals with this stream
                pasteIntoTree(parent, stream);
            } else if (tr
                    .isDataFlavorSupported(TransferableNode.plainTextFlavor)) {
                Object stream = TransferableNode.plainTextFlavor
                        .getReaderForText(tr);
                // deals with this stream
                pasteIntoTree(parent, stream);
            }
        } catch (IOException io) {
            io.printStackTrace();
        } catch (UnsupportedFlavorException ufe) {
            ufe.printStackTrace();
        }
        // closes the compound edit
        edit.end();
    }

    /**
     * pastes in parent from input. Input can be String, Reader or InputStream
     * 
     * @param parent the element where to paste the content of the clipboard
     * @param stream the input from where to get the data
     */
    private void pasteIntoTree(ElementImpl parent, Object stream) {
        // gets the data as an InputSource
        InputSource source;
        if (stream instanceof InputStream) {
            source = new InputSource((InputStream) stream);
        } else if (stream instanceof Reader) {
            source = new InputSource((Reader) stream);
        } else if (stream instanceof String) {
            source = new InputSource(new StringReader((String) stream));
        } else {
            // unknown type, thus does nothing
            msgDis.popupMessage("Unable to paste");
            msgDis
                    .displayMessage("This editor is enable to paste the type of data presentin the "
                            + "clipboard.\n");
            return;
        }
        // retrieve a document from the String data
        Document document = this.getNodeFromInputSource(source);
        if (document == null) {
            msgDis.popupMessage("Unable to paste");
            msgDis
                    .displayMessage("The Editor was unable to paste the content of the clpiboard into "
                            + " this tree.\nYou probably tried to paste bad-formed xml.\n");
            return;
        }
        // begins a compoundedit for undo management since several nodes
        // are removed at once
        CompoundEdit edit = new CompoundEdit(tree.getTreeView().getEditor());
        tree.getTreeView().getEditor().getUndoManager().addEdit(edit);
        // pastes every node of the new document in parent. Only redisplays
        // for the last node
        NodeList nodeList = document.getChildNodes();
        int maxIndex = nodeList.getLength();
        for (int i = 0; i < maxIndex; i++) {
            parent.pasteNode((TreeElement) nodeList.item(i), i == maxIndex - 1);
        }
        // closes the compound edit
        edit.end();
    }

    /**
     * called each time a paste into parent has to be performed in this tree.
     * The content of the clipboard will be copied into this tree
     */
    public void pasteIntoParent() {
        TreePath[] treePaths = tree.getSelectionPaths();
        if (treePaths == null) {
            msgDis.popupMessage("Paste canceled");
            msgDis
                    .displayMessage("There was no selected node to tell me where to paste.\n");
            return;
        }
        XmlTreeNode parent = null;
        for (int i = 0; i < treePaths.length; i++) {
            XmlTreeNode parentCandidate = ((XmlTreeNode) treePaths[i]
                    .getLastPathComponent()).getParentTreeNode();
            if (parentCandidate == null) {
                msgDis.popupMessage("Paste canceled");
                msgDis
                        .displayMessage("You tried to paste into the parent of a node that appeared"
                                + " to have no parent.\n");
                return;
            }
            if (parent == null) {
                parent = parentCandidate;
            } else if (parent != parentCandidate) {
                msgDis.popupMessage("Paste canceled");
                msgDis
                        .displayMessage("Several nodes are currently selected and they don't have a"
                                + " common parent. Thus, I don't know where to paste.\n");
                return;
            }
        }
        if (parent.getNode() instanceof ElementImpl) {
            this.internalPaste(parent);
        }
    }

    /**
     * called each time a paste has to be performed in this tree. The content of
     * the clipboard will be copied into this tree
     * 
     * @param parent the treeNode where to paste the content of the clipBoard
     */
    private void internalPaste(XmlTreeNode parent) {
        // check we're pasting into an element
        if (!(parent.getNode() instanceof ElementImpl)) {
            msgDis.popupMessage("Paste canceled");
            msgDis.displayMessage("You can only paste inside an element.\n");
            return;
        }
        ElementImpl parentNode = (ElementImpl) parent.getNode();
        // check that we are not pasting into a reference
        if (parent.isReference()) {
            msgDis.popupMessage("Paste canceled");
            msgDis
                    .displayMessage("You cannot paste a node inside a reference.\n");
            return;
        }
        this.pasteIntoTree(parentNode);
    }

    /**
     * called each time a paste into selected node has to be performed in this
     * tree. The content of the clipboard will be copied into this tree
     */
    public void paste() {
        TreePath[] treePaths = tree.getSelectionPaths();
        if (treePaths == null) {
            msgDis.popupMessage("Paste canceled");
            msgDis
                    .displayMessage("There was no selected node to tell me where to paste.\n");
            return;
        }
        if (treePaths.length > 1) {
            msgDis.popupMessage("Paste canceled");
            msgDis
                    .displayMessage("Several nodes are currently selected. Thus, I don't know where"
                            + " to paste.\n");
            return;
        }
        XmlTreeNode parent = (XmlTreeNode) treePaths[0].getLastPathComponent();
        if (parent != null) {
            this.internalPaste(parent);
        }
    }

}