/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese;

import java.awt.Component;
import java.awt.Image;
import java.beans.BeanDescriptor;
import java.beans.BeanInfo;
import java.beans.Beans;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.beans.PropertyDescriptor;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.beanbuilder.BeanInfoFactory;
import javax.swing.beanbuilder.JarClassLoader;

import com.simulogic.cese.properties.ModelPropertyDescriptor;
import com.javable.cese.templates.AbstractAPModel;

/**
 * Encapsulates a model item that can be instantiated A model also knows how to
 * instantiate itself.
 */
public class ModelInfo {

	private AbstractAPModel model;

	private String beanName;

	private Class beanClass;

	private BeanInfo info;

	private BeanDescriptor descriptor;

	private ModelPropertyDescriptor[] descriptors;

	private SimulationHistory history;

	private boolean modified;

	private final PropertyChangeSupport pcs = new PropertyChangeSupport(this);

	/**
	 * Exporter for this environment
	 */
	public static DataExport export = new DataExport();

	/**
	 * Default constructor
	 */
	public ModelInfo() {
		history = new SimulationHistory(this);
	}

	/**
	 * Constructs a model item from the fully qualified bean name
	 * 
	 * @param beanName
	 *            the name of the bean that this item represents.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public ModelInfo(String beanName) throws IOException,
			ClassNotFoundException {
		this.beanName = beanName;

		beanClass = Class.forName(beanName);
		info = BeanInfoFactory.getBeanInfo(beanClass);
		descriptor = info.getBeanDescriptor();

		newModel();
	}

	/**
	 * Constructs a model item from a serialized bean.
	 * 
	 * @param obj
	 *            an instance of the deserialized bean.
	 * @param beanName
	 *            the name of the resouce without the extension
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public ModelInfo(Object obj, String beanName) throws IOException,
			ClassNotFoundException {
		this.beanName = beanName;

		beanClass = obj.getClass();
		info = BeanInfoFactory.getBeanInfo(beanClass);
		descriptor = info.getBeanDescriptor();

		// Should tweak the contents of the BeanDescriptor since the
		// BeanInfo will reflect the base class
		descriptor.setName(beanName);
		descriptor.setShortDescription("Serialized Object: " + beanName);

		String shortName = beanName;
		int ix = beanName.lastIndexOf('.');
		if (ix >= 0) {
			shortName = beanName.substring(ix + 1);
		}
		descriptor.setDisplayName(shortName);

		newModel();
	}

	/**
	 * Constructs a model item from a class.
	 * 
	 * @param cls
	 *            a valid java bean class.
	 * @throws IOException
	 * @throws ClassNotFoundException
	 */
	public ModelInfo(Class cls) throws IOException, ClassNotFoundException {
		beanClass = cls;
		beanName = cls.getName();

		info = BeanInfoFactory.getBeanInfo(beanClass);
		descriptor = info.getBeanDescriptor();

		newModel();
	}

	/**
	 * Creates a new bean from the encapsulated class. Sets some initial values
	 * 
	 * @exception java.lang.ClassNotFoundException
	 * @exception java.io.IOException
	 * @see java.beans.Beans
	 */
	public void newModel() throws IOException, ClassNotFoundException {
		setModel(Beans
				.instantiate(JarClassLoader.getJarClassLoader(), beanName));

		history = new SimulationHistory(this);
	}

	/**
	 * Returns current model
	 * 
	 * @return model
	 */
	public AbstractAPModel getModel() {
		return model;
	}

	/**
	 * Sets new model
	 * 
	 * @param b
	 *            new model
	 */
	public void setModel(Object b) {
		model = ((AbstractAPModel) Beans
				.getInstanceOf(b, AbstractAPModel.class));
		model.setExporter(export);
	}

	/**
	 * Returns the name of this bean
	 * 
	 * @return bean name
	 */
	public String getBeanName() {
		return beanName;
	}

	/**
	 * Returns the class of this bean
	 * 
	 * @return bean class
	 */
	public Class getBeanClass() {
		return beanClass;
	}

	/**
	 * Returns the beaninfo of this bean
	 * 
	 * @return bean beaninfo
	 */
	public BeanInfo getBeanInfo() {
		return info;
	}

	/**
	 * Returns the beandescriptor of this bean
	 * 
	 * @return bean beandescriptor
	 */
	public BeanDescriptor getBeanDescriptor() {
		return descriptor;
	}

	/**
	 * Retrieves the icon for the beaninfo
	 * 
	 * @param type -
	 *            One of BeanInfo.ICON_
	 * @return bean icon
	 */
	public Image getIcon(int type) {
		return info.getIcon(type);
	}

	/**
	 * Returns the bean name from descriptor
	 * 
	 * @return bean name
	 */
	public String getName() {
		return descriptor.getName();
	}

	/**
	 * Returns the bean display name from the descriptor
	 * 
	 * @return bean display name
	 */
	public String getDisplayName() {
		return descriptor.getDisplayName();
	}

	/**
	 * Returns the bean short description from the descriptor
	 * 
	 * @return bean short description
	 */
	public String getShortDescription() {
		return descriptor.getShortDescription();
	}

	/**
	 * Returns a flag indicating if the encapsulated object has a customizer
	 * 
	 * @return true if has customizer
	 */
	public boolean hasCustomizer() {
		if (descriptor != null) {
			Class cls = descriptor.getCustomizerClass();
			return (cls != null);
		}

		return false;
	}

	/**
	 * Gets the customizer for the current object
	 * 
	 * @return New instance of the customizer or null if there isn't a
	 *         customizer
	 */
	public Component getCustomizer() {
		Component customizer = null;

		if (descriptor != null) {
			Class cls = descriptor.getCustomizerClass();

			if (cls != null) {
				try {
					customizer = (Component) cls.newInstance();
				} catch (Exception ex) {
				}
			}
		}

		return customizer;
	}

	/**
	 * Gets the beans <code>PropertyDescriptor</code>s.
	 * 
	 * @return An array of PropertyDescriptors describing the editable
	 *         properties supported by this bean. May return null if the
	 *         information should be obtained by automatic analysis.
	 */
	public ModelPropertyDescriptor[] getModelPropertyDescriptors() {
		if (descriptors == null) {
			ArrayList list = new ArrayList();
			list.addAll(Arrays.asList(info.getPropertyDescriptors()));

			descriptors = new ModelPropertyDescriptor[list.size()];
			for (int i = 0; i < descriptors.length; i++) {
				descriptors[i] = new ModelPropertyDescriptor(
						(PropertyDescriptor) list.get(i));
			}
		}

		return descriptors;
	}

	/**
	 * Returns simulation history
	 * 
	 * @return simulation history
	 */
	public SimulationHistory getHistory() {
		return history;
	}

	/**
	 * Sets new simulation history
	 * 
	 * @param history
	 *            simulation history
	 */
	public void setHistory(SimulationHistory history) {
		this.history = history;
	}

	/**
	 * Returns modified status
	 * 
	 * @return modified status
	 */
	public boolean isModified() {
		return modified;
	}

	/**
	 * Sets modified status
	 * 
	 * @param modified
	 *            modified status
	 */
	public void setModified(boolean modified) {
		pcs.firePropertyChange("modified", this.modified,
				this.modified = modified);
	}

	public void addPropertyChangeListener(PropertyChangeListener l) {
		pcs.addPropertyChangeListener(l);
	}

	public void removePropertyChangeListener(PropertyChangeListener l) {
		pcs.removePropertyChangeListener(l);
	}
}