/*
 * com.simulogic.cese -- Cell Electrophysiology Simulation Environment (CESE)
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 * 
 * Dalhousie Univ., Halifax, NS, Canada
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.cese.properties;

import java.beans.PropertyDescriptor;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.ListIterator;
import java.util.Vector;

import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;

import com.simulogic.cese.ModelInfo;
import com.simulogic.cese.ResourceManager;

/**
 * Model for the export tree
 */
public class ExportTreeModel extends DefaultTreeModel {

	private java.util.Vector groups;

	private ModelPropertyDescriptor[] descriptors;

	private ModelInfo info;

	// Sort options
	/**
	 * No sort
	 */
	public static final int SORT_DEF = 0;

	/**
	 * Sort by name
	 */
	public static final int SORT_NAME = 1;

	/**
	 * Sort by type
	 */
	public static final int SORT_TYPE = 2;

	// Shared instance of a Comparator
	private static DescriptorComparator dscComparator = new DescriptorComparator();

	private static GroupComparator grpComparator = new GroupComparator();

	/**
	 * Default constructor
	 */
	public ExportTreeModel() {
		super(new DefaultMutableTreeNode(ResourceManager.getResource("Export")));
		groups = new Vector();
	}

	/**
	 * Set the tree model to represent the properties of the model
	 * 
	 * @param info
	 *            model info
	 */
	public void setModelInfo(ModelInfo info) {
		this.info = info;

		if (info != null) {
			filterTree();
		}
	}

	/**
	 * Checks if node is a leaf
	 * 
	 * @param node
	 *            node to check
	 * @return true if node is a leaf
	 */
	public boolean isLeaf(Object node) {
		return (node instanceof PropertyDescriptor);
	}

	/**
	 * Returns child for the object
	 * 
	 * @param parent
	 *            parent object
	 * @param index
	 *            index in parent branch
	 * @return child
	 */
	public Object getChild(Object parent, int index) {
		if (parent.equals(root))
			return groups.get(index);
		return getGroup((String) parent).get(index);
	}

	/**
	 * Returns childs count
	 * 
	 * @param parent
	 *            parent object
	 * @return child count
	 */
	public int getChildCount(Object parent) {
		if (parent.equals(root))
			return groups.size();
		return getGroup((String) parent).size();
	}

	/**
	 * Returns index of child
	 * 
	 * @param parent
	 *            parent object
	 * @param child
	 *            child object
	 * @return child index
	 */
	public int getIndexOfChild(Object parent, Object child) {
		if (parent.equals(root)) {
			for (int i = 0; i < groups.size(); i++) {
				if (groups.get(i) == (String) child)
					return i;
			}
		}
		for (int i = 0; i < getGroup((String) parent).size(); i++) {
			if (getGroup((String) parent).get(i).equals(child))
				return i;
		}
		return -1;
	}

	private Vector getGroup(String grp) {
		Vector gr = new Vector();
		for (int i = 0; i < descriptors.length; i++) {
			PropertyDescriptor pd = descriptors[i].getDescriptor();
			if (grp == pd.getShortDescription())
				gr.addElement(pd);
		}
		return gr;
	}

	/**
	 * Sorts the tree according to the sort type.
	 * 
	 * @param sort
	 *            sort type
	 */
	public void sortTree(int sort) {
		if (sort == SORT_DEF || descriptors == null)
			return;

		if (sort == SORT_NAME) {
			Arrays.sort(descriptors, dscComparator);
		} else {
			Arrays.sort(descriptors, dscComparator);
		}
		Collections.sort(groups, grpComparator);

		nodeStructureChanged(root);
	}

	/**
	 * Filters the tree to display only properties with specific attributes.
	 */
	public void filterTree() {
		if (info == null)
			return;

		groups = new Vector();

		// Use collections to filter out unwanted properties
		ArrayList list = new ArrayList();
		list.addAll(Arrays.asList(info.getModelPropertyDescriptors()));

		ListIterator iterator = list.listIterator();
		ModelPropertyDescriptor desc;
		while (iterator.hasNext()) {
			desc = (ModelPropertyDescriptor) iterator.next();

			if (!desc.getDescriptor().getDisplayName().startsWith("|OUT")
					|| desc.getDescriptor().isHidden()) {
				iterator.remove();
			} else {
				if (!groups
						.contains(desc.getDescriptor().getShortDescription()))
					groups.addElement(desc.getDescriptor()
							.getShortDescription());
			}

		}

		descriptors = new ModelPropertyDescriptor[list.size()];
		for (int i = 0; i < descriptors.length; i++) {
			descriptors[i] = (ModelPropertyDescriptor) list.get(i);
		}

		sortTree(SORT_NAME);
	}
}