/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.cursors;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;
import javax.swing.tree.DefaultMutableTreeNode;

import com.simulogic.dataview.DataChannel;
import com.simulogic.dataview.DataGroup;
import com.simulogic.dataview.DataStorage;
import com.simulogic.dataview.DataView;
import com.simulogic.dataview.ResourceManager;
import com.simulogic.dataview.TimeChannel;
import com.simulogic.dataview.analysis.ChannelStatsProxy;

/**
 * Class that puts up data cursors
 * 
 * @see DataView
 */
public class CursorProxy implements javax.swing.event.TreeModelListener,
		javax.swing.event.ChangeListener {

	private DataView view;

	private CursorSlider cursors;

	private DataStorage cursorStorage;

	private Vector stats;

	private JMenu popup;

	private JCheckBoxMenuItem visible = null;

	private JCheckBoxMenuItem linked = null;

	private JCheckBoxMenuItem locked = null;

	/**
	 * Default constructor
	 * 
	 * @param v
	 *            data view
	 */
	public CursorProxy(DataView v) {
		view = v;

		view.getScrollPane().getViewport().addChangeListener(this);
		view.getStorage().addTreeModelListener(this);

		cursors = new CursorSlider(4, view);
		cursors.addMouseListener(view);
		cursors.addMouseMotionListener(view);

		clearCursorStorage();
	}

	/**
	 * Adds a pair of cursors
	 */
	public void addCursorsPair() {
		cursors.addThumb();
		cursors.addThumb();
	}

	/**
	 * Returns current cursors
	 * 
	 * @return cursors
	 */
	public CursorSlider getCursorSlider() {
		return cursors;
	}

	private void changeLimits() {
		if ((view.getStorage() == null)
				|| (view.getStorage().getGroupsSize() == 0))
			return;

		double min = Math.min(view.getStorage().getLimitRectangle().getX(),
				view.getScale().getX());
		double max = Math.max(view.getStorage().getLimitRectangle().getX()
				+ view.getStorage().getLimitRectangle().getWidth(), view
				.getScale().getX()
				+ view.getScale().getWidth());

		java.awt.Dimension size = cursors.getSize();
		java.awt.Dimension thumb = ((CursorSliderAdditional) cursors.getUI())
				.getThumbSize();

		int width = view.getPreferredSize().width + thumb.width;
		int height = view.getPreferredSize().height;

		if ((size.width != width) || (size.height != height))
			cursors.setPreferredSize(new Dimension(width, height));

		cursors.setMinimum(min);
		cursors.setMaximum(max);
	}

	private void changeRange() {
		changeLimits();
		cursors.distributeThumbs(cursors.getMinimumLimit(), cursors
				.getMaximumLimit());
	}

	/**
	 * Distributes cursors in the current view
	 */
	public void bringCursorsHere() {
		cursors.distributeThumbs(view.getScale().getX(), view.getScale().getX()
				+ view.getScale().getWidth());
	}

	/**
	 * Returns storage for cursor data
	 * 
	 * @return cursor storage
	 */
	public DataStorage getCursorsStorage() {
		return cursorStorage;
	}

	/**
	 * Clears cursors data in cursor storage
	 */
	public void clearCursorStorage() {
		com.simulogic.dataview.TimeChannel xres = null;
		com.simulogic.dataview.DataChannel yres = null;

		stats = new Vector();

		if (cursorStorage == null)
			cursorStorage = new DataStorage();

		cursorStorage.setRoot(new DefaultMutableTreeNode(ResourceManager
				.getResource("Legend")));
		int c = cursors.getThumbNum();
		if (c > 1)
			stats.add(new ChannelStatsProxy());
		if (c > 3)
			stats.add(new ChannelStatsProxy());

		Vector labels = new Vector();
		labels.add("Trace");
		labels.add("X");
		labels.add("Y");
		labels.add("\u0394X");
		labels.add("\u0394Y");
		labels.add("Xmin");
		labels.add("Ymin");
		labels.add("Xmax");
		labels.add("Ymax");
		labels.add("Slope");
		labels.add("Mean");
		labels.add("Std. Dev.");

		xres = new TimeChannel((String) labels.get(0), 0);
		xres.setRate(1.0);

		DataGroup group = new DataGroup(ResourceManager.getResource("Cursors")
				+ " " + cursorStorage.getGroupsSize(), "");
		group.addChannel(xres);

		for (int i = 1; i < labels.size(); i++) {
			for (int j = 0; j < c; j++) {

				if (i < 3) {
					yres = new DataChannel(labels.get(i) + " " + (j + 1), 0);
					group.addChannel(yres);
				} else {

					if (j == 1) {
						yres = new DataChannel(labels.get(i) + " " + "1..2", 0);
						group.addChannel(yres);
					}

					if (j == 3) {
						yres = new DataChannel(labels.get(i) + " " + "3..4", 0);
						group.addChannel(yres);
					}
				}
			}
		}

		cursorStorage.addGroup(group);
	}

	/**
	 * Appends cursors data in cursor storage
	 */
	public void appendCursorStorage() {
		if (!cursors.isVisible())
			return;

		for (int i = 0; i < stats.size(); i++) {
			((ChannelStatsProxy) stats.get(i)).clear();
		}

		DataChannel channel;
		int c = cursors.getThumbNum();

		DataChannel s = view.getStorage().getSelectedChannel();
		DataChannel x = view.getStorage().getXChannel(s);

		for (int i = 0; i < cursorStorage.getChannelsSize(0); i++) {
			channel = cursorStorage.getChannel(0, i);
			channel.grow(1);

			if (i > 0) {
				if (i <= c) {
					channel.setData(channel.size() - 1, cursors
							.getValueAt(i - 1));
				}

				if ((i > c * 2) && (i <= c * 2 + c / 2)) {
					if (Math.IEEEremainder(i, 2.0) != 0.0) {
						channel.setData(channel.size() - 1, getStat(1, 1, x, s,
								((ChannelStatsProxy) stats.get(0))));
					} else {
						channel.setData(channel.size() - 1, getStat(1, 2, x, s,
								((ChannelStatsProxy) stats.get(1))));
					}
				}

				if (cursors.isLocked() && (s != null)
						&& (s.getAttribute().isNormal())
						&& (!view.getStorage().isXChannel(s))) {
					if ((i > c) && (i <= c * 2)) {
						channel.setData(channel.size() - 1,
								com.simulogic.dataview.analysis.ChannelStats
										.getValueAtX(cursors.getValueAt(i - c
												- 1), x, s));
					}

					if (i > c * 2) {
						if (Math.IEEEremainder(i, 2.0) != 0.0) {
							channel.setData(channel.size() - 1, getStat(i - c
									* 2, 1, x, s, ((ChannelStatsProxy) stats
									.get(0))));
						} else {
							channel.setData(channel.size() - 1, getStat(
									(i - c * 2) - 1, 2, x, s,
									((ChannelStatsProxy) stats.get(1))));
						}

					}
				}
			}

		}

		cursorStorage.updateModel(0);
	}

	private double getStat(int type, int region, DataChannel xchannel,
			DataChannel ychannel, ChannelStatsProxy proxy) {

		double xmin;
		double xmax;

		// Cursors 1..2, 3..4
		xmin = cursors.getValueAt(region * 2 - 2);
		xmax = cursors.getValueAt(region * 2 - 1);

		if (type == 1)
			return xmax - xmin;
		if (type == 3)
			return com.simulogic.dataview.analysis.ChannelStats.getValueAtX(xmax,
					xchannel, ychannel)
					- com.simulogic.dataview.analysis.ChannelStats.getValueAtX(
							xmin, xchannel, ychannel);

		if (xmin > xmax) {
			double swap = xmin;
			xmin = xmax;
			xmax = swap;
		}

		if (type == 5)
			return proxy.getNegativePeakTime(xmin, xmax, xchannel, ychannel);
		if (type == 7)
			return proxy.getNegativePeak(xmin, xmax, xchannel, ychannel);
		if (type == 9)
			return proxy.getPositivePeakTime(xmin, xmax, xchannel, ychannel);
		if (type == 11)
			return proxy.getPositivePeak(xmin, xmax, xchannel, ychannel);

		if (type == 13)
			return proxy.getAbsoluteSlope(xmin, xmax, xchannel, ychannel);

		if (type == 15)
			return proxy.getMean(xmin, xmax, xchannel, ychannel);
		if (type == 17)
			return proxy.getStandardDeviation(xmin, xmax, xchannel, ychannel);

		return 0.0;
	}

	/**
	 * Creates cursor-related popup menu
	 * 
	 * @return popup menu
	 */
	public JMenu createPopup() {
		JMenuItem m_menu = null;

		if (popup == null) {
			try {
				popup = new JMenu(ResourceManager.getResource("Cursors"));
				visible = new JCheckBoxMenuItem(ResourceManager
						.getResource("Show"));
				visible.setSelected(cursors.isVisible());
				visible.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						cursors.setVisible(((JCheckBoxMenuItem) e.getSource())
								.isSelected());
					}
				});
				popup.add(visible);

				popup.addSeparator();

				m_menu = new JMenuItem(ResourceManager
						.getResource("Bring_Here"));
				m_menu.setAccelerator(KeyStroke.getKeyStroke(
						java.awt.event.KeyEvent.VK_SPACE, 1));
				m_menu.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						bringCursorsHere();
					}
				});
				popup.add(m_menu);

				linked = new JCheckBoxMenuItem(ResourceManager
						.getResource("Link_to_Partner"));
				linked.setSelected(cursors.isLinked());
				linked.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						cursors.setLinked(((JCheckBoxMenuItem) e.getSource())
								.isSelected());
					}
				});
				popup.add(linked);

				locked = new JCheckBoxMenuItem(ResourceManager
						.getResource("Lock_to_Data"));
				locked.setSelected(cursors.isLocked());
				locked.addActionListener(new ActionListener() {

					public void actionPerformed(ActionEvent e) {
						cursors.setLocked(((JCheckBoxMenuItem) e.getSource())
								.isSelected());
						cursors.repaint();
					}
				});
				popup.add(locked);

			} catch (Exception e) {
			}
		} else {
			visible.setSelected(cursors.isVisible());
			linked.setSelected(cursors.isLinked());
			locked.setSelected(cursors.isLocked());
		}
		return popup;
	}

	// / --- interfaces

	/**
	 * @param evt
	 */
	public void stateChanged(javax.swing.event.ChangeEvent evt) {
		if (evt.getSource() == view.getScrollPane().getViewport())
			changeLimits();
	}

	/**
	 * @param evt
	 */
	public void treeStructureChanged(javax.swing.event.TreeModelEvent evt) {
	}

	/**
	 * @param evt
	 */
	public void treeNodesInserted(javax.swing.event.TreeModelEvent evt) {
		if (view.getStorage().getGroupsSize() == 1)
			changeRange();
	}

	/**
	 * @param evt
	 */
	public void treeNodesRemoved(javax.swing.event.TreeModelEvent evt) {
		if (view.getStorage().getGroupsSize() == 1)
			changeRange();
	}

	/**
	 * @param evt
	 */
	public void treeNodesChanged(javax.swing.event.TreeModelEvent evt) {
	}

}