/*
 * com.simulogic.dataview -- The Java2D Plotting Framework
 *
 * Copyright (C) 1999 - 2005 Sergey Missan
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.simulogic.dataview.plots;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;

import javax.swing.Icon;
import javax.swing.ImageIcon;

import com.simulogic.dataview.ResourceManager;
import com.simulogic.dataview.DataChannel;

/**
 * Plot that uses filled/empty bars to render plot data
 */
public class ColumnPlot extends AbstractPlot {

    private Point2D.Double tp;
    private Point2D.Double ta;

    /**
     * Creates new ColumnPlot
     */
    public ColumnPlot() {
        super();
        distance = .5;
    }

    /**
     * Plots content of the single channel
     * 
     * @param g2 graphics
     * @param xChannel channel containing independent variable values
     * @param yChannel channel containing dependent variable values
     * @param xMin minimal value for plotting range
     * @param xMax maximal value for plotting range
     * @param transform plot transform
     */
    public final void plotChannel(Graphics2D g2, DataChannel xChannel,
            DataChannel yChannel, double xMin, double xMax,
            AffineTransform transform) {
        try {
            if (yChannel.getAttribute().isNormal()) {
                g2.setStroke(yChannel.getAttribute().getStroke());
                double width = (Math.abs(xChannel.maxLimit() - xChannel.minLimit()) / xChannel.size())
                        * yChannel.getAttribute().getSpace();
                Color frame = yChannel.getAttribute().getColor();
                Color fill = yChannel.getAttribute().getFillColor();
                Rectangle2D.Double bounds = new Rectangle2D.Double();
                Area col = new Area();
                
                tp = new Point2D.Double();
                ta = new Point2D.Double();
                AffineTransform d_tr = transform.createInverse();
                tp .setLocation(0, 0);
                d_tr.transform(tp, tp);
                ta .setLocation(distance, 0);
                d_tr.transform(ta, ta);
                double dist_x = ta.distance(tp);
                ta .setLocation(0, distance);
                d_tr.transform(ta, ta);
                double dist_y = ta.distance(tp);
                
                tp = null;

                for (int k = 0; k < xChannel.size(); k++) {
                    double xval = xChannel.getData(k);
                    if ((xval >= xMin) && (xval <= xMax)
                            && (k < yChannel.size())) {
                        double yval = yChannel.getData(k);
                        if (tp == null) {
                            tp = new Point2D.Double(xval, yval);
                            ta = new Point2D.Double(xval, yval);

                            bounds.setFrame(xval - width / 2.0,
                                    Math.min(yval, 0), width, Math.abs(yval));
                            col = new Area(bounds.getFrame());
                            g2.setColor(fill);
                            g2.fill(col.createTransformedArea(transform));
                            g2.setColor(frame);
                            g2.draw(col.createTransformedArea(transform));
                        } else {
                            tp.setLocation(xval, yval);

                            if ((Math.abs(ta.x-tp.x) >= dist_x) || (Math.abs(ta.y-tp.y) >= dist_y)) {
                                bounds.setFrame(xval - width / 2.0, Math.min(
                                        yval, 0), width, Math.abs(yval));
                                col = new Area(bounds.getFrame());
                                g2.setColor(fill);
                                g2.fill(col.createTransformedArea(transform));
                                g2.setColor(frame);
                                g2.draw(col.createTransformedArea(transform));

                                ta.setLocation(xval, yval);
                            }
                        }
                    }
                }
            }
        } catch (Exception e) {
        }
    }

    /**
     * Determines whether point is located within a plot
     * 
     * @param x point x coordinate
     * @param y point y coordinate
     * @param xChannel channel containing independent variable values
     * @param yChannel channel containing dependent variable values
     * @param transform plot transform
     * 
     * @return true if point belongs to channel
     */
    public boolean isChannelPoint(double x, double y, DataChannel xChannel,
            DataChannel yChannel, AffineTransform transform) {
        Point2D.Double dstTop = new Point2D.Double();
        Point2D.Double dstBtm = new Point2D.Double();

        try {
            transform.inverseTransform(new Point2D.Double(x, y - hotspot),
                    dstTop);

            transform.inverseTransform(new Point2D.Double(x, y + hotspot),
                    dstBtm);

            double tst = com.simulogic.dataview.analysis.ChannelStats
                    .getValueAtX(dstTop.getX(), xChannel, yChannel);

            if ((tst > dstBtm.getY()) && (tst < dstTop.getY())) { return true; }
        } catch (Exception e) {
        }
        return false;
    }

    /**
     * Returns new instance of the plot
     * 
     * @return new instance of the current plot
     */
    public AbstractPlot createPlot() {
        return new ColumnPlot();
    }
    
    /**
     * Returns plot name.
     * 
     * @return plot name.
     */
    public String getName() {
    	return ResourceManager.getResource("Column_plot");
    }
    
    /**
     * Returns small icon for plot.
     * 
     * @return mall icon for plot.
     */
    public Icon getSmallIcon() {
    	return new ImageIcon(this.getClass().getResource("columnplot_image"));
    }
}